/*
QUSOFT MICROSYSTMES
1869

Classe: TMainMenu - Type MainMenu

Hirarchie:
TObject -> TComponent -> TContainer -> TFrame -> TMainMenu
*/

/*
		Copyright 2001 Frdric Brown
*/

/*
    This file is part of Gundealer.

    Gundealer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gundealer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gundealer; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * TMainMenu class represents the displaye when the game is started.
*/

#define SystemMenu_OPTION_NO_SAVE 0
#define SystemMenu_OPTION_NORMAL 1

#define SystemMenu_CHOICE_NONE -1
#define SystemMenu_CHOICE_QUIT 2
#define SystemMenu_CHOICE_LOAD 3
#define SystemMenu_CHOICE_RESTART 4
#define SystemMenu_CHOICE_CONTINUE 5
#define SystemMenu_CHOICE_SAVE 6

//Structure de l'objet de Type MainMenu  extension de TFrame
typedef struct MainMenu  {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Frame*	(*clone)		(struct Frame*);
	BOOL					(*equals)		(struct Frame*, struct Object*);
	void					(*finalize)	(struct MainMenu*);
	TString* 			(*toString) (struct Frame*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)		 	(struct Frame*);
	void							(*setBounds)	(struct Container*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Container*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TVector* contentPane;	//Le contenu du Container
	BOOL* visibility;			//La visibilit des composants
	void							(*bringOn)		(struct Container*, TComponent*);
	void							(*center)			(struct Container*, TComponent*);
	void							(*drag)				(struct Container*, TComponent*, TCursor*);
	TString* title;				//Le titre de la fentre
	void							(*setTitle)		(struct Frame*, TString*);
	void							(*add)				(struct Container*, TComponent*);
	TString *strLoading;
	TString *strLoad;
	TString *strSave;
	TString *strNull;
	TLabel *line1;
	TLabel *line2;
	TGarbageCollector* gc;
} TMainMenu ;

//Dclaration des constructeurs
struct MainMenu* TMainMenu0();

//Dclaration des mthodes
void 	TMainMenu_finalize		(TMainMenu*);

/**
 * Ce constructeur instancie un objet de Type SystemMenu.
 @since Release 1.0
*/
TMainMenu* TMainMenu0 () {
	TMainMenu *this = (TMainMenu*)malloc(sizeof(TMainMenu));
	
	int i;
	
	this->class  = "TMainMenu";
	this->super  = "TFrame";
	
	this->height 			= 75;
	this->visible 		= FALSE;
	this->width 			= 220;
	this->x						= 0;
	this->y						= 0;
	this->img					= (BITMAP*)malloc(sizeof(BITMAP));
	this->contentPane = TVector0();
	this->visibility	= NULL;
	this->title				= s("1869");
	
	this->clone 	 = TFrame_clone;
	this->equals 	 = TFrame_equals;
	this->finalize = TMainMenu_finalize;
	this->toString = TFrame_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TFrame_print;
	this->setBounds  = TContainer_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TContainer_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	
	this->bringOn		 = TContainer_bringOn;
	this->center		 = TContainer_center;
	this->drag			 = TContainer_drag;
	this->add				 = TContainer_add;

	this->setTitle	 = TFrame_setTitle;
	
	this->gc				 = TGarbageCollector0();

	this->strLoading = s("Loading ...");
	this->gc->add(this->gc, this->strLoading);
	this->strLoad = s("Loading game ...");
	this->gc->add(this->gc, this->strLoad);
	this->strSave = s("Saving game ...");
	this->gc->add(this->gc, this->strSave);
	this->strNull = s("");
	this->gc->add(this->gc, this->strNull);

	this->line1		 = TLabel0();
	this->gc->add(this->gc, this->line1);
	this->line2		 = TLabel0();
	this->gc->add(this->gc, this->line2);

	//this->setBounds(this, 0, 0, 220, 75);

	this->line1->move(this->line1, 1, 30);
	this->add(this, this->line1);

	this->line2->move(this->line2, 1, 45);
	this->add(this, this->line2);

	return this;
}

void TMainMenu_finalize (TMainMenu* this)
{
	this->setVisible(this, FALSE);
	
	TRY {
		this->contentPane->finalize(this->contentPane);
		this->gc->finalize(this->gc);
		if (this->visibility != NULL) {
			free(this->visibility);
		}
		this->title->finalize(this->title);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}
