/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TButton - Type Button

Hirarchie:
TObject -> TComponent -> TCaptionedComponent -> TButton
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TButton reprsente un bouton sur lequel l'utilisateur peut cliquer.
 @since Release 1.0
*/

//Structure de l'objet de Type Button extension de TCaptionedComponent
typedef struct Button {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Button*	(*clone)		(struct Button*);
	BOOL						(*equals)		(struct CaptionedComponent*, struct Object*);
	void						(*finalize)	(struct Button*);
	TString* 				(*toString) (struct CaptionedComponent*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Button*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TString* caption;	//L'tiquette du composant
	void							(*setCaption)	(struct CaptionedComponent*, TString*);
} TButton;

//Dclaration des constructeurs
TButton* TButton0();
TButton* TButton1(TString*);
TButton* TButton2(char*);

//Dclaration des mthodes
TButton*							TButton_clone				(TButton*);
void									TButton_finalize		(TButton*);
void									TButton_print				(TButton*);

/**
 * Ce constructeur instancie un objet de Type Button.
 @since Release 1.0
*/
TButton* TButton0 () {
	TButton *this = (TButton*)malloc(sizeof(TButton));
	
	this->class  = "TButton";
	this->super  = "TCaptionedComponent";
	
	this->height 	= 15;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->caption	= s("");
	
	this->clone 	 = TButton_clone;
	this->equals 	 = TCaptionedComponent_equals;
	this->finalize = TButton_finalize;
	this->toString = TCaptionedComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TButton_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	this->setCaption = TCaptionedComponent_setCaption;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type Button en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TButton* TButton1 (TString* caption) {
	
	TButton *this = TButton0();
	
	this->setCaption(this, caption);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type Button en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TButton* TButton2 (char* caption) {
	
	TButton *this;
	TString *string;
	
	string = s(caption);
	
	this = TButton1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type Button qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TButton* TButton_clone(TButton* this) {

	return (TButton*)TCaptionedComponent_clone(this);
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TButton_finalize (TButton* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->caption->finalize(this->caption);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TButton_print(TButton* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	
	for (i=y0; i<y1; i++) {
		DrawLine(x0, i, x1, i, A_REVERSE);
	}
	
	DrawLine(x0, y0, x1, y0, A_NORMAL);
	DrawLine(x0, y1, x1, y1, A_NORMAL);
	DrawLine(x0, (y0+1), x0, (y1-1), A_NORMAL);
	DrawLine(x1, (y0+1), x1, (y1-1), A_NORMAL);
	
	DrawStr((x0+2), (y0+2), this->caption->string, A_NORMAL);
}
