/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TNode - Type Node

Hirarchie:
TObject -> TNode
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TNode reprsente un noeud pouvant tre li dynamiquement.
 @since Release 1.0
*/

//Structure de l'objet de Type Node extension de TObject
typedef struct Node {
	char *class;     //Le nom de la classe
	char *super;     //Le nom de la superclasse
	struct Object* 	(*clone) 		(struct Object*);
	BOOL 						(*equals) 	(struct Node*, struct Node*);
	void 						(*finalize)	(struct Node*);
	TString* 				(*toString) (struct Object*);
	void *element;   //L'lment reli  ce neud
	void *after;     //Le noeud suivant
} TNode;

//Dclaration des constructeurs
TNode* TNode0	(void*);
TNode* TNode1	(void*, void*);

//Dclaration des mthodes
BOOL TNode_equals 	(TNode*, TNode*);
void TNode_finalize	(TNode*);

/**
 * Ce constructeur instancie un objet de Type Node.
 @param element L'lment li  ce noeud.
 @since Release 1.0
*/
TNode* TNode0 (void* element) {
	TNode *this = (TNode*)malloc(sizeof(TNode));
	
	this->class   = "TNode";
	this->super   = "TObject";
	this->element = element;
	this->after   = NULL;
	
	this->clone    = TObject_clone;
	this->equals   = TNode_equals;
	this->finalize = TNode_finalize;
	this->toString = TObject_toString;
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type Node.
 @param element L'lment li  ce noeud.
 @param after L'lment suivant ce noeud.
 @since Release 1.0
*/
TNode* TNode1 (void* element, void* after) {
	TNode *this;
	
	this = TNode0(element);
	
	this->after = after;
	
	return this;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TNode_finalize (TNode* this) {

		TRY {
			free(this);	
		}
		ONERR {}
		ENDTRY
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Pour la classe TNode, elle compare l'lment li au noeud  l'objet en
 * paramtres.
 @param anotherNode L'objet avec qui comparer.
 @return equal - Vrai si les objets sont gaux.
 @since Release 1.0
*/
BOOL TNode_equals (TNode* this, TNode* node) {
	
	BOOL result;
	
	result = (this->element == node->element);
	
	return result;
}
