/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TTextArea - Type TextArea

Hirarchie:
TObject -> TComponent -> TCaptionedComponent -> TTextField -> TTextArea
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TTextArea reprsente une zone de texte.  Une TextArea est en fait
 * un TextField avec la possibilit de rpartir le texte sur plusieurs lignes.
 @since Release 1.0
*/

//Structure de l'objet de Type TextField extension de TCaptionedComponent
typedef struct TextArea {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct TextArea*	(*clone)		(struct TextArea*);
	BOOL							(*equals)		(struct CaptionedComponent*, struct Object*);
	void							(*finalize)	(struct TextArea*);
	TString* 					(*toString) (struct CaptionedComponent*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct TextArea*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TString* caption;	//L'tiquette du composant
	void							(*setCaption)	(struct TextArea*, TString*);
	BOOL masked;			//Indique si le texte est masqu
	char mask;				//Le masque
	void							(*setMasked)	(struct TextField*, BOOL);
	void							(*setMask)		(struct TextField*, char);
	void							(*use)				(struct TextArea*);
	BOOL editable;		//Indique si le texte est ditable
} TTextArea;

//Dclaration des constructeurs
TTextArea* TTextArea0();
TTextArea* TTextArea1(TString*);
TTextArea* TTextArea2(char*);

//Dclaration des mthodes
TTextArea*						TTextArea_clone				(TTextArea*);
void									TTextArea_finalize		(TTextArea*);
void									TTextArea_print				(TTextArea*);
void									TTextArea_use					(TTextArea*);
void									TTextArea_setCaption	(TTextArea*, TString*);

/**
 * Ce constructeur instancie un objet de Type TextArea.
 @since Release 1.0
*/
TTextArea* TTextArea0 () {
	TTextArea *this = (TTextArea*)malloc(sizeof(TTextArea));
	
	this->class  = "TTextArea";
	this->super  = "TTextField";
	
	this->height 	 = 15;
	this->visible  = FALSE;
	this->width 	 = 0;
	this->x				 = 0;
	this->y				 = 0;
	this->img			 = (BITMAP*)malloc(sizeof(BITMAP));
	this->caption	 = s("");
	this->masked	 = FALSE;
	this->mask		 = '*';
	this->editable = TRUE;
	
	this->clone 	 = TTextArea_clone;
	this->equals 	 = TCaptionedComponent_equals;
	this->finalize = TTextArea_finalize;
	this->toString = TCaptionedComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TTextArea_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	this->setCaption = TTextArea_setCaption;

	this->use				= TTextArea_use;
	this->setMasked = TTextField_setMasked;
	this->setMask		= TTextField_setMask;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type TextArea en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TTextArea* TTextArea1 (TString* caption) {
	
	TTextArea *this = TTextArea0();
	
	this->setCaption(this, caption);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type TextArea en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TTextArea* TTextArea2 (char* caption) {
	
	TTextArea *this;
	TString *string;
	
	string = s(caption);
	
	this = TTextArea1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type TextArea qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TTextArea* TTextArea_clone(TTextArea* this) {

	TTextArea *clone;
	
	clone = TTextArea0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->caption	= c(this->caption);
	clone->masked		= this->masked;
	clone->mask			= this->mask;
	clone->editable = this->editable;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TTextArea_finalize (TTextArea* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->caption->finalize(this->caption);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TTextArea_print(TTextArea* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	int lineNum = 0;
	int maxLength;
	int maxLines;
	int cptH = 0;
	int cptW = 0;
	TString *line;
	TString *text;
	BOOL end = FALSE;
	
	for (i=y0; i<y1; i++) {
		DrawLine(x0, i, x1, i, A_REVERSE);
	}
	
	DrawLine(x0, y0, x1, y0, A_NORMAL);
	DrawLine(x0, y1, x1, y1, A_NORMAL);
	DrawLine(x0, (y0+1), x0, (y1-1), A_NORMAL);
	DrawLine(x1, (y0+1), x1, (y1-1), A_NORMAL);
	
	text = this->caption->clone(this->caption);
	
	if (this->masked) {
		for (i = 0; i < text->length(text); i++) {
			text->string[i] = this->mask;
		}
	}
	
	cptW = CaptionedComponent_getFontWidth();
	cptH = CaptionedComponent_getFontHeight();
	
	maxLength = this->width / cptW - 1;
	
	maxLines = this->height / (cptH + 2);
	
	for (lineNum = 0; (lineNum < maxLines) && !end; lineNum++) {
		if (((lineNum + 1) * maxLength - 1) < text->length(text)) {
			line = text->substring0(text, lineNum * maxLength, (lineNum + 1) * maxLength - 1);
		}
		else {
			line = text->substring0(text, lineNum * maxLength, text->length(text) - 1);
			end = TRUE;
		}
		
		DrawStr((x0+2), ((lineNum * (cptH + 2)) + (y0+2)), line->string, A_NORMAL);
		line->finalize(line);
	}
	
	text->finalize(text);
}

/**
 * Cette mthode permet  l'utilisateur d'utiliser le composant.
 @since Release 1.0
*/
void TTextArea_use(TTextArea* this) {
	
	int key;
	TString *caption = s("");
	char *array;
	int maxLength;
	int maxLines;
	int ptr;
	int cptW;
	int cptH;
	
	if (!this->editable) {
		return;
	}
	
	cptW = CaptionedComponent_getFontWidth();
	cptH = CaptionedComponent_getFontHeight();
	
	maxLength = (this->width - 4) / cptW;
	
	maxLines = this->height / (cptH + 2);
	
	array = (char*)malloc((maxLength * (maxLines + 1)) + 1);
	
	strcpy(array, this->caption->string);
	
	ptr = this->caption->length(this->caption);
	
	do {
		key = ngetchx();
		
		if (key >= ' ' && key <= '~' && ptr < (maxLength * (maxLines + 1))) {
			array[ptr] = key;
			ptr++;
			array[ptr] = '\0';
		}
		else if (key == KEY_BACKSPACE) {
			if (ptr > 0) {
				array[ptr-1] = '\0';
				ptr--;
			}
		}
		else if (key == KEY_CLEAR) {
			array[0] = '\0';
			ptr = 0;
		}
		
		caption->finalize(caption);
		caption = s(array);
		
		this->setVisible(this, FALSE);
		this->caption = c(caption);
		this->setVisible(this, TRUE);
	}	while (key != KEY_ENTER);
	
	free(array);
}

/**
 * Cette mthode change l'tiquette du composant.   la diffrence
 * d'un CaptionedComponent, les dimensions d'un TextArea sont immuables.
 @param caption L'tiquette du composant
 @since Release 1.0
*/
void TTextArea_setCaption(TTextArea* this, TString* caption) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->caption->finalize(this->caption);
	this->caption = c(caption);
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}
